<?php

/**
 * LaterPay appearance controller.
 *
 * Plugin Name: LaterPay
 * Plugin URI: https://github.com/laterpay/laterpay-wordpress-plugin
 * Author URI: https://laterpay.net/
 */
class LaterPay_Controller_Admin_Appearance extends LaterPay_Controller_Admin_Base {

    /**
     * @see LaterPay_Core_Event_SubscriberInterface::get_subscribed_events()
     */
    public static function get_subscribed_events() {
        return array(
            'wp_ajax_laterpay_appearance' => array(
                array( 'laterpay_on_admin_view', 200 ),
                array( 'process_ajax_requests' ),
                array( 'laterpay_on_ajax_user_can_activate_plugins', 200 ),
            ),
            'laterpay_admin_enqueue_scripts' => array(
                array( 'laterpay_on_admin_view', 200 ),
                array( 'laterpay_on_plugin_is_active', 200 ),
                array( 'add_custom_styles' )
            ),
        );
    }

    /**
     * Add appearance styles
     */
    public function add_custom_styles() {
        // apply colors config
        LaterPay_Helper_Appearance::add_overlay_styles( 'laterpay-admin' );
    }

    /**
     * @see LaterPay_Core_View::load_assets()
     */
    public function load_assets() {
        parent::load_assets();

        // Get data for GA.
        $merchant_key = LaterPay_Controller_Admin::get_merchant_id_for_ga();

        LaterPay_Controller_Admin::register_common_scripts( 'appearance' );

        // load page-specific JS
        wp_register_script(
            'laterpay-backend-appearance',
            $this->config->js_url . '/laterpay-backend-appearance.js',
            array( 'jquery', 'laterpay-common' ),
            $this->config->version,
            true
        );
        wp_enqueue_script( 'laterpay-backend-appearance' );

        wp_localize_script(
            'laterpay-backend-appearance',
            'lpVars',
            array(
                'overlaySettings'    => wp_json_encode(
                    array(
                        'default' => LaterPay_Helper_Appearance::get_default_options(),
                        'current' => LaterPay_Helper_Appearance::get_current_options()
                    )
                ),
                'l10n_print_after'   => 'lpVars.overlaySettings = JSON.parse(lpVars.overlaySettings)',
                'gaData'             => array(
                    'sandbox_merchant_id' => ( ! empty( $merchant_key ) ) ? $merchant_key : '',
                ),
                'invalidConfigError' => esc_html__( 'Please select one of the recommended options above to ensure that your users can purchase all content types.', 'laterpay' ),
            )
        );
    }

    /**
     * @see LaterPay_Core_View::render_page()
     */
    public function render_page() {
        $this->load_assets();

        $menu = LaterPay_Helper_View::get_admin_menu();

        $view_args = array(
            'plugin_is_in_live_mode'              => $this->config->get( 'is_in_live_mode' ),
            'appearance_obj'                      => $this,
            'admin_menu'                          => add_query_arg( LaterPay_Helper_Request::laterpay_encode_url_params( array( 'page' => $menu['account']['url'] ) ), admin_url( 'admin.php' ) ),
            'purchase_button_positioned_manually' => get_option( 'laterpay_purchase_button_positioned_manually' ),
            'time_passes_positioned_manually'     => get_option( 'laterpay_time_passes_positioned_manually' ),
            'overlay'                             => LaterPay_Helper_Appearance::get_current_options(),
        );

        $this->assign( 'laterpay', $view_args );

        $this->render( 'backend/appearance' );
    }

    /**
     * Process Ajax requests from appearance tab.
     *
     * @param LaterPay_Core_Event $event
     * @throws LaterPay_Core_Exception_InvalidIncomingData
     * @throws LaterPay_Core_Exception_FormValidation
     *
     * @return void
     */
    public static function process_ajax_requests( LaterPay_Core_Event $event ) {
        $event->set_result(
            array(
                'success' => false,
                'message' => __( 'An error occurred when trying to save your settings. Please try again.', 'laterpay' )
            )
        );

        $submitted_form_value = filter_input( INPUT_POST, 'form', FILTER_SANITIZE_STRING );

        if ( null === $submitted_form_value ) {
            // invalid request
            throw new LaterPay_Core_Exception_InvalidIncomingData( 'form' );
        }

        if ( function_exists( 'check_admin_referer' ) ) {
            check_admin_referer( 'laterpay_form' );
        }

        switch ( $submitted_form_value ) {
            case 'appearance_config':
                // Update Appearance tab config.
                $appearance_config_options = [
                    'lp_show_purchase_overlay'              => 'show_purchase_overlay',
                    'lp_show_purchase_button_above_article' => 'show_purchase_button_above_article',
                    'lp_show_tp_sub_below_modal'            => 'show_tp_sub_below_modal',
                    'lp_show_introduction'                  => 'show_introduction',
                ];

                // Create an array of keys to iterate over.
                $appearance_config_keys = array_keys( $appearance_config_options );
                foreach ( $appearance_config_keys as $key ) {
                    $config_option_value = filter_input( INPUT_POST, $appearance_config_options[ $key ], FILTER_SANITIZE_STRING );

                    // check if value is set and set option value accordingly.
                    $appearance_config_options[ $key ] = null === $config_option_value ? 0 : absint( $config_option_value );
                }

                update_option( 'lp_appearance_config', $appearance_config_options );

                $lp_body_text    = [];
                $body_text_value = filter_input( INPUT_POST, 'show_body_text', FILTER_SANITIZE_STRING );
                if ( null !== $body_text_value ) {
                    $lp_body_text['enabled'] = absint( $body_text_value );
                } else {
                    $lp_body_text['enabled'] = 0;
                }

                $custom_html_text = esc_html__( 'Enter custom HTML here', 'laterpay' );
                if ( 1 === $lp_body_text['enabled'] ) {
                    $body_text_content_value = filter_input( INPUT_POST, 'body_text_content', FILTER_SANITIZE_FULL_SPECIAL_CHARS );
                    $lp_body_text['content'] = ! empty( $body_text_content_value ) ? $body_text_content_value : "<p>$custom_html_text</p>";
                    update_option( 'lp_body_text', $lp_body_text );
                } else {
                    $laterpay_body_text      = get_option( 'lp_body_text' );
                    $lp_body_text['content'] = ! empty( $laterpay_body_text['content'] ) ? $laterpay_body_text['content'] : "<p>$custom_html_text</p>";
                    update_option( 'lp_body_text', $lp_body_text );
                }

                // Update purchase header.
                $purchase_header = filter_input( INPUT_POST, 'purchase_header', FILTER_SANITIZE_STRING );
                update_option( 'laterpay_overlay_header_title', ! empty( $purchase_header ) ? $purchase_header : __( 'Read now, pay later', 'laterpay' ) );

                // Update existing options.
                $existing_options = [
                    'laterpay_purchase_button_positioned_manually' => 'is_purchase_button_custom_positioned',
                    'laterpay_time_passes_positioned_manually'     => 'is_tp_sub_custom_positioned',
                    'laterpay_overlay_show_footer'                 => 'show_footer',
                ];

                // Create an array of keys to iterate over.
                $existing_options_keys = array_keys( $existing_options );

                // check if value is set and set option value accordingly.
                foreach ( $existing_options_keys as $key ) {
                    $existing_option_value = filter_input( INPUT_POST, $existing_options[ $key ], FILTER_SANITIZE_STRING );
                    update_option( $key, null === $existing_option_value ? 0 : absint( $existing_option_value ) );
                }

                $event->set_result(
                    array(
                        'success' => true,
                        'message' => __( 'Appearance settings saved successfully.', 'laterpay' )
                    )
                );

                break;
            case 'overlay_settings':

                // handle additional settings save if present in request
                $header_background_color = filter_input( INPUT_POST, 'header_background_color', FILTER_SANITIZE_STRING );
                $background_color        = filter_input( INPUT_POST, 'background_color', FILTER_SANITIZE_STRING );
                $main_text_color         = filter_input( INPUT_POST, 'main_text_color', FILTER_SANITIZE_STRING );
                $description_text_color  = filter_input( INPUT_POST, 'description_text_color', FILTER_SANITIZE_STRING );
                $button_background_color = filter_input( INPUT_POST, 'button_background_color', FILTER_SANITIZE_STRING );
                $button_hover_color      = filter_input( INPUT_POST, 'button_hover_color', FILTER_SANITIZE_STRING );
                $button_hover_text_color = filter_input( INPUT_POST, 'button_hover_text_color', FILTER_SANITIZE_STRING );
                $button_text_color       = filter_input( INPUT_POST, 'button_text_color', FILTER_SANITIZE_STRING );
                $link_main_color         = filter_input( INPUT_POST, 'link_main_color', FILTER_SANITIZE_STRING );
                $link_hover_color        = filter_input( INPUT_POST, 'link_hover_color', FILTER_SANITIZE_STRING );
                $footer_background_color = filter_input( INPUT_POST, 'footer_background_color', FILTER_SANITIZE_STRING );

                update_option( 'laterpay_overlay_header_bg_color', $header_background_color );
                update_option( 'laterpay_overlay_main_bg_color', $background_color );
                update_option( 'laterpay_overlay_main_text_color', $main_text_color );
                update_option( 'laterpay_overlay_description_color', $description_text_color );
                update_option( 'laterpay_main_color', $button_background_color );
                update_option( 'laterpay_hover_color', $button_hover_color );
                update_option( 'laterpay_overlay_button_hover_text_color', $button_hover_text_color );
                update_option( 'laterpay_overlay_button_text_color', $button_text_color );
                update_option( 'laterpay_overlay_link_main_color', $link_main_color );
                update_option( 'laterpay_overlay_link_hover_color', $link_hover_color );
                update_option( 'laterpay_overlay_footer_bg_color', $footer_background_color );

                $event->set_result(
                    array(
                        'success' => true,
                        'message' => __( 'Color scheme saved successfully.', 'laterpay' )
                    )
                );

                break;

            default:
                break;
        }
    }

    /**
     * Render overlay
     */
    public function render_overlay() {

        $config = laterpay_get_plugin_config();

        $additional_data = array(
            'currency' => $config->get( 'currency.code' ),
            'icons'    => $config->get_section( 'payment.icons' )
        );

        $this->assign( 'overlay', array_merge( LaterPay_Helper_Appearance::get_current_options(), $additional_data ) );

        $this->render( 'backend/partials/purchase-overlay' );
    }
}
